//
// Module: OffCCDPSFCorrector
// Author: Richard Saxton
//
// Description: 
//
// Calculates the number of pixels in the source area which lie
// outside the live CCD area
//
// Modifications: See ChangeLog
// 
// $Id: OffCCDPSFCorrector.cc,v 1.3 2005/12/21 11:09:55 rsaxton Exp $

/* General */
#include <cmath>

/* SAS-specific */
#include "Tracer.h"
#include "DataFilter.h"
#include "Xmm.h"

/* Local */
#include "NumericalRange.h"
#include "ArfDataServer.h"
#include "OffCCDPSFCorrector.h"
#include "OffCCDPixelIterator.h"
#include "DetmapFilter.h"
#include "MissingPixCorrector.h"

/* Constructor */
OffCCDPSFCorrector::OffCCDPSFCorrector( unsigned eefactor, PsfModel * psf):
  MissingPixCorrector(eefactor),
  _psf(psf)
{
  PROC;
  ENDPROC;
}

/* Destructor */
OffCCDPSFCorrector::~OffCCDPSFCorrector()
{
  PROC;
  ENDPROC;
}

double OffCCDPSFCorrector::eeFraction( double energy  ) const
{
  PROC;

  
  double fracenclosed   = 0;
  double contrib = 0;

  _psf->energy(energy);

  FOREACH(_pxitr)
    {
      PixelElement pixel = _pxitr->currentItem();
      
      
     /* Add contribution to PSF */
      contrib = _psf->int_psf( pixel.xCentre(),
			      pixel.yCentre(),
			      pixel.xWidth(),
			      pixel.yWidth());
      
      errstr << "OffCCDPSFCorrector: Pixel details  " << pixel << endl
	     << " PSF contribution " << contrib << endl
	     << Sas::message(Msg::App, Msg::Noisy);
      
      fracenclosed += contrib;
    }
  
  errstr << "OffCCDPSFCorrector: Fraction of psf enclosed " 
	 << fracenclosed << endl
	 << Sas::message(Msg::App, Msg::Verbose); 

  RETURN fracenclosed;
}
  
/*
   Method: filterMap()
         : Get a pixel iterator for the Off CCD pixels
*/
void OffCCDPSFCorrector::filterMap()
{
  PROC;

  /* Create an iterator for the pixels which lie within the source region
     but off the chips. NB: Currently have to convert circles to polygons 
     to be compatible with Span function */

          _pxitr = new OffCCDPixelIterator(
                   arfDataServer->PolyRegion(),
                   arfDataServer->extent(),
                   arfDataServer->positionUtils(),
                   arfDataServer->eventFlags(),
                   arfDataServer->ignoreOutofFOV(),
                   arfDataServer->ccdMask());

  ENDPROC;
}

int OffCCDPSFCorrector::totalPixels() const
{
  PROC;

   /* Calculate the total number of detector pixels covered by the bad 
      pixels */

     int total = UNT(_pxitr->size() * _pxitr->wcsconvertor()->xWidth() *
                                  _pxitr->wcsconvertor()->yWidth());

  RETURN total;
}

