//
// Module: PointEeCorrector
// Author: Hassan Siddiqui
//
// Description: 
//
// Point source implementation of an Encircled Energy Corrector class
//
// Modifications: See ChangeLog
// 
// $Id: PointEeCorrector.cc,v 1.1.1.1 2004/02/09 14:54:12 eojero Exp $

/* General */

/* SAS-specific */
#include "Tracer.h"

/* Local */
#include "NumericalRange.h"
#include "PointEeCorrector.h"
#include "PixelIterator.h"
#include "ArfDataServer.h"
#include "CalServer.h"

/* Constructor */
PointEeCorrector::PointEeCorrector(unsigned eefactor):
  IterEeCorrector(eefactor)
{
  PROC;
  ENDPROC;
}

/* Destructor */
PointEeCorrector::~PointEeCorrector()
{
  PROC;
  ENDPROC;
}

double PointEeCorrector::eeFraction( double energy  ) const
{
  PROC;

  
  double fracenclosed   = 0;
  double contrib = 0;

      errstr << "PointEeCorrector: calles"
	     << Sas::message(Msg::App, Msg::Verbose);
      
 /* Get the PSF function and an iterator for its integration from the 
    ARF data server */

  PixelIterator * pxitr = arfDataServer->PsfPixelIterator();
  PsfModel * psf = arfDataServer->psf();

 /* Set the energy for this PSF model */
  psf->energy(energy);

 /* Integrate the PSF contribution */
  FOREACH(pxitr)
    {
      PixelElement pixel = pxitr->currentItem();
      
      
      /* Add contribution to PSF. Have to treat the circular PSF differently */
      if (calServer.state()->accuracyLevel()->get() == EXTENDED)
      {
         contrib = psf->rad_psf();
      }
      else 
      {
         contrib = psf->int_psf( pixel.xCentre(),
			      pixel.yCentre(),
			      pixel.xWidth(),
			      pixel.yWidth());
      }
      
      errstr << "PointEeCorrector: Pixel details  " << pixel << endl
	     << " PSF contribution " << contrib << endl
	     << Sas::message(Msg::App, Msg::Noisy);
      
      fracenclosed += contrib;
    }
  
  errstr << "PointEeCorrector: Fraction of psf enclosed " 
	 << fracenclosed << endl
	 << Sas::message(Msg::App, Msg::Verbose); 

  RETURN fracenclosed;
}
  
